#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Example experiment of a dumbbell topology in which a Rapid sender 
# and a cross traffic sender share a single, bottleneck link.  The 
# cross traffic sender sends in CBR bursts.
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Define experiment parameters.
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Experiment parameters.
set start_rapid 			"0.0";		# seconds
set start_ct 				"1.0";		# seconds
set end_experiment			"5.0";		# seconds
set rtt					0.125;	# seconds
set router_queue_limit 			"5000000"; 	# packets
set out_filename 			"| gzip > data/out.gz"
set bottleneck_capacity 		"100Mbps"
set link_speed 				"1000Mbps"
set link_delay 				[expr $rtt / 6];
set ns [new Simulator]

# Rapid parameters.
set tao 				"0.050"; 	# seconds
set eta 				"0.045"; 	# seconds
set rapid_pkt_size 			"1000B"
set rapid_max_p_stream_size		"90"; 		# packets

# On-off cross traffic parameters.
# r_avg = r_on * t_on / (t_on + t_off)
set t_on 				"0.100"; 	# seconds
set t_off 				"0.200"; 	# seconds
set r_on 				"30Mbps"
set ct_pkt_size 			"1000B"

# Cleanup function called to end the experiment.
proc finish {} {
	global ns out_file
	$ns flush-trace
	close $out_file
	exit 0
}


#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Create shared routers and bottleneck link.
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

set r1 [$ns node]; # Node 0
set r2 [$ns node]; # Node 1

# Shared, bottleneck link.
$ns duplex-link $r1 $r2 $bottleneck_capacity $link_delay DropTail
$ns queue-limit $r1 $r2 $router_queue_limit
$ns queue-limit $r2 $r1 $router_queue_limit


#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Create RAPID traffic.
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Create sender node and link to r1.
set sender_node [$ns node]; # Node 2
$ns duplex-link $sender_node $r1 $link_speed $link_delay DropTail
$ns queue-limit $sender_node $r1 $router_queue_limit
$ns queue-limit $r1 $sender_node $router_queue_limit

# Create Rapid TCP source at sender node.
set rapid_source [new Agent/TCP/Rapid]
$rapid_source set tao $tao
$rapid_source set eta $eta
$rapid_source set pkt_size $rapid_pkt_size
$rapid_source set max_p_stream_size $rapid_max_p_stream_size
$ns attach-agent $sender_node $rapid_source

# Create application data source.
set application [new Application/FTP]
$application attach-agent $rapid_source
$application set type_ FTP

# Create destination node and link to r2.
set receiver_node [$ns node]; # Node 3
$ns duplex-link $r2 $receiver_node $link_speed $link_delay DropTail
$ns queue-limit $r2 $receiver_node $router_queue_limit
$ns queue-limit $receiver_node $r2 $router_queue_limit

# Create RAPID TCP sink at destination node.
set rapid_sink [new Agent/TCPSink/RapidSink]
$ns attach-agent $receiver_node $rapid_sink
	
$ns connect $rapid_source $rapid_sink


#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Create cross traffic nodes.
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Create cross traffic sender node and connect to r1.
set ct_sender_node [$ns node]; # Node 4
$ns duplex-link $ct_sender_node $r1 $link_speed $link_delay DropTail
$ns queue-limit $ct_sender_node $r1 $router_queue_limit
$ns queue-limit $r1 $ct_sender_node $router_queue_limit

# Create cross traffic UDP source at source node.
set ct_source [new Agent/UDP]
$ns attach-agent $ct_sender_node $ct_source

# Create cross traffic application data source
set ct_application [new Application/Traffic/CBR]
$ct_application set PacketSize_ $ct_pkt_size
$ct_application set rate_ $r_on
$ct_application attach-agent $ct_source

# Create cross traffic destination node and connect to r2.
set ct_receiver_node [$ns node]; # Node 5
$ns duplex-link $r2 $ct_receiver_node $link_speed $link_delay DropTail
$ns queue-limit $r2 $ct_receiver_node $router_queue_limit
$ns queue-limit $ct_receiver_node $r2 $router_queue_limit

# Create cross traffic UDP sink at destination node.
set ct_sink [new Agent/Null]
$ns attach-agent $ct_receiver_node $ct_sink

$ns connect $ct_source $ct_sink


#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Trace queue data at the bottleneck link.
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


set out_file [open $out_filename w]
$ns trace-queue $r1 $r2 $out_file


#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Schedule events and run the experiment!
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Start the start of the simulation.
$ns at $start_rapid "$application start"
	
# Set the on-off periods for the cross traffic.
set i $start_ct
while { $i < $end_experiment } {
	$ns at [expr $i] "$ct_application start"
	set i [expr $i+$t_on]
	$ns at [expr $i] "$ct_application stop"
	set i [expr $i+$t_off]
}

# Set the end of the simulation.
$ns at $end_experiment "finish"

# Run the experiment!
$ns run
